from pyspark.sql.types import ArrayType, StringType, StructField, StructType


# noinspection PyPep8Naming
class SubstancePolymer:
    @staticmethod
    def get_schema(recursion_depth: int = 0) -> StructType:
        # from https://hl7.org/FHIR/patient.html
        from spark_fhir_schemas.r4.complex_types.id import id
        from spark_fhir_schemas.r4.complex_types.meta import Meta
        from spark_fhir_schemas.r4.complex_types.uri import uri
        from spark_fhir_schemas.r4.complex_types.code import code
        from spark_fhir_schemas.r4.complex_types.narrative import Narrative
        from spark_fhir_schemas.r4.complex_types.resourcelist import ResourceList
        from spark_fhir_schemas.r4.complex_types.extension import Extension
        from spark_fhir_schemas.r4.complex_types.codeableconcept import CodeableConcept
        from spark_fhir_schemas.r4.complex_types.substancepolymer_monomerset import SubstancePolymer_MonomerSet
        from spark_fhir_schemas.r4.complex_types.substancepolymer_repeat import SubstancePolymer_Repeat
        if recursion_depth > 3:
            return StructType([])
        schema = StructType(
            [
                StructField("resourceType", StringType(), True),
                StructField("id", id.get_schema(recursion_depth + 1), True),
                StructField(
                    "meta", Meta.get_schema(recursion_depth + 1), True
                ),
                StructField(
                    "implicitRules", uri.get_schema(recursion_depth + 1), True
                ),
                StructField(
                    "language", code.get_schema(recursion_depth + 1), True
                ),
                StructField(
                    "text", Narrative.get_schema(recursion_depth + 1), True
                ),
                StructField(
                    "contained",
                    ArrayType(ResourceList.get_schema(recursion_depth + 1)),
                    True
                ),
                StructField(
                    "extension",
                    ArrayType(Extension.get_schema(recursion_depth + 1)), True
                ),
                StructField(
                    "modifierExtension",
                    ArrayType(Extension.get_schema(recursion_depth + 1)), True
                ),
                StructField(
                    "class", CodeableConcept.get_schema(recursion_depth + 1),
                    True
                ),
                StructField(
                    "geometry",
                    CodeableConcept.get_schema(recursion_depth + 1), True
                ),
                StructField(
                    "copolymerConnectivity",
                    ArrayType(CodeableConcept.get_schema(recursion_depth + 1)),
                    True
                ),
                StructField("modification", ArrayType(StringType()), True),
                StructField(
                    "monomerSet",
                    ArrayType(
                        SubstancePolymer_MonomerSet.
                        get_schema(recursion_depth + 1)
                    ), True
                ),
                StructField(
                    "repeat",
                    ArrayType(
                        SubstancePolymer_Repeat.
                        get_schema(recursion_depth + 1)
                    ), True
                ),
            ]
        )

        return schema
